#include "LED.cpp"
#include "Timer.cpp"
#include <Arduino.h>
#include <BLEDevice.h>
#include <BLEServer.h>
#include <BLEUtils.h>

#define FADE_IN_DURATION 500
#define FADE_OUT_DURATION 2000

#define SERVICE_UUID "4fafc201-1fb5-459e-8fcc-c5c9c331914b"
#define CHARACTERISTIC_UUID "beb5483e-36e1-4688-b7f5-ea07361b26a8"

class BluetoothCallback : public BLECharacteristicCallbacks {
	LED *_led;
	void onWrite(BLECharacteristic *pCharacteristic) {
		std::string value = pCharacteristic->getValue();
		if (value.length() > 0) {
			Serial.println(value.c_str());
			if (value == "on")
				_led->on();
			else
				_led->off();
		}
	}

	public:

	BluetoothCallback(LED* led){
		_led = led;
	}

};

LEDOutput _ledOutput(0);
LED _led(&_ledOutput, FADE_IN_DURATION, FADE_OUT_DURATION);

void toggle(){
	//_led.toggle();
}

Timer _timer(3000, &toggle);

void setup() {
	Serial.begin(115200);
	_ledOutput.attach(16);
	_ledOutput.attach(17);
	_ledOutput.attach(5);
	_ledOutput.attach(18);
	_ledOutput.attach(23);

	BLEDevice::init("Van Lights");
	BLEServer *pServer = BLEDevice::createServer();
	BLEService *pService = pServer->createService(SERVICE_UUID);
	BLECharacteristic *pCharacteristic = pService->createCharacteristic(CHARACTERISTIC_UUID, BLECharacteristic::PROPERTY_READ | BLECharacteristic::PROPERTY_WRITE);

	pCharacteristic->setCallbacks(new BluetoothCallback(&_led));

	pCharacteristic->setValue("Hello World");
	pService->start();

	BLEAdvertising *pAdvertising = pServer->getAdvertising();
	pAdvertising->start();
}

void loop() {
	_led.loop();
	_timer.loop();
}
